"""
Content blocks are for building complex, nested HTML structures that usually
contain sub-blocks, and may require javascript to function properly.
"""
from django.utils.translation import gettext_lazy as _
from wagtail.core import blocks
from wagtail.documents.blocks import DocumentChooserBlock
from wagtail.images.blocks import ImageChooserBlock
from wagtail.snippets.blocks import SnippetChooserBlock

from .base_blocks import BaseBlock, BaseLayoutBlock, ButtonMixin, CollectionChooserBlock
from .html_blocks import ButtonBlock


class CardBlock(BaseBlock):
    """
    A component of information with image, text, and buttons.
    """
    image = ImageChooserBlock(
        required=False,
        max_length=255,
        label=_('Image'),
    )
    title = blocks.CharBlock(
        required=False,
        max_length=255,
        label=_('Title'),
    )
    subtitle = blocks.CharBlock(
        required=False,
        max_length=255,
        label=_('Subtitle'),
    )
    description = blocks.RichTextBlock(
        features=['bold', 'italic', 'ol', 'ul', 'hr', 'link', 'document-link'],
        label=_('Body'),
    )
    links = blocks.StreamBlock(
        [('Links', ButtonBlock())],
        blank=True,
        required=False,
        label=_('Links'),
    )

    class Meta:
        template = 'coderedcms/blocks/card_foot.html'
        icon = 'fa-list-alt'
        label = _('Card')


class CarouselBlock(BaseBlock):
    """
    Enables choosing a Carousel snippet.
    """
    carousel = SnippetChooserBlock('coderedcms.Carousel')

    class Meta:
        icon = 'image'
        label = _('Carousel')
        template = 'coderedcms/blocks/carousel_block.html'


class ImageGalleryBlock(BaseBlock):
    """
    Show a collection of images with interactive previews that expand to
    full size images in a modal.
    """
    collection = CollectionChooserBlock(
        required=True,
        label=_('Image Collection'),
    )

    class Meta:
        template = 'coderedcms/blocks/image_gallery_block.html'
        icon = 'image'
        label = _('Image Gallery')


class ModalBlock(ButtonMixin, BaseLayoutBlock):
    """
    Renders a button that then opens a popup/modal with content.
    """
    header = blocks.CharBlock(
        required=False,
        max_length=255,
        label=_('Modal heading'),
    )
    content = blocks.StreamBlock(
        [],
        label=_('Modal content'),
    )
    footer = blocks.StreamBlock(
        [
            ('text', blocks.CharBlock(icon='fa-file-text-o', max_length=255, label=_('Simple Text'))),  # noqa
            ('button', ButtonBlock()),
        ],
        required=False,
        label=_('Modal footer'),
    )

    class Meta:
        template = 'coderedcms/blocks/modal_block.html'
        icon = 'fa-window-maximize'
        label = _('Modal')


class NavBaseLinkBlock(BaseBlock):
    display_text = blocks.CharBlock(
        required=False,
        max_length=255,
        label=_('Display text'),
    )
    image = ImageChooserBlock(
        required=False,
        label=_('Image'),
    )


class NavExternalLinkBlock(NavBaseLinkBlock):
    """
    External link.
    """
    link = blocks.CharBlock(
        required=False,
        label=_('URL'),
    )

    class Meta:
        template = 'coderedcms/blocks/external_link_block.html'
        label = _('External Link')


class NavPageLinkBlock(NavBaseLinkBlock):
    """
    Page link.
    """
    page = blocks.PageChooserBlock(
        label=_('Page'),
    )

    class Meta:
        template = 'coderedcms/blocks/page_link_block.html'
        label = _('Page Link')


class NavDocumentLinkBlock(NavBaseLinkBlock):
    """
    Document link.
    """
    document = DocumentChooserBlock(
        label=_('Document'),
    )

    class Meta:
        template = 'coderedcms/blocks/document_link_block.html'
        label = _('Document Link')


class NavSubLinkBlock(BaseBlock):
    """
    Streamblock for rendering nested sub-links.
    """
    sub_links = blocks.StreamBlock(
        [
            ('page_link', NavPageLinkBlock()),
            ('external_link', NavExternalLinkBlock()),
            ('document_link', NavDocumentLinkBlock()),
        ],
        required=False,
        label=_('Sub-links'),
    )


class NavExternalLinkWithSubLinkBlock(NavSubLinkBlock, NavExternalLinkBlock):
    """
    Extermal link with option for sub-links.
    """
    class Meta:
        label = _('External link with sub-links')


class NavPageLinkWithSubLinkBlock(NavSubLinkBlock, NavPageLinkBlock):
    """
    Page link with option for sub-links or showing child pages.
    """
    show_child_links = blocks.BooleanBlock(
        required=False,
        default=False,
        label=_('Show child pages'),
        help_text=_('Automatically show a link to the Page’s child pages as a dropdown menu.'),
    )

    class Meta:
        label = _('Page link with sub-links')


class NavDocumentLinkWithSubLinkBlock(NavSubLinkBlock, NavDocumentLinkBlock):
    """
    Document link with option for sub-links.
    """
    class Meta:
        label = _('Document link with sub-links')


class PriceListItemBlock(BaseBlock):
    """
    Represents one item in a PriceListBlock, such as an entree in a restaurant menu.
    """
    image = ImageChooserBlock(
        required=False,
        label=_('Image'),
    )
    name = blocks.CharBlock(
        required=True,
        max_length=255,
        label=_('Name'),
    )
    description = blocks.TextBlock(
        required=False,
        rows=4,
        label=_('Description'),
    )
    price = blocks.CharBlock(
        required=True,
        label=_('Price'),
        help_text=_('Any text here. Include currency sign if desired.'),
    )

    class Meta:
        template = 'coderedcms/blocks/pricelistitem_block.html'
        icon = 'fa-usd'
        label = _('Price List Item')


class PriceListBlock(BaseBlock):
    """
    A price list, such as a menu for a restaurant.
    """
    heading = blocks.CharBlock(
        required=False,
        max_length=255,
        label=_('Heading'),
    )
    items = blocks.StreamBlock(
        [
            ('item', PriceListItemBlock()),
        ],
        label=_('Items'),
    )

    class Meta:
        template = 'coderedcms/blocks/pricelist_block.html'
        icon = 'fa-usd'
        label = _('Price List')


class ContentWallBlock(BaseBlock):
    """
    Enables choosing a ContentWall snippet.
    """
    content_wall = SnippetChooserBlock('coderedcms.ContentWall')
    show_content_wall_on_children = blocks.BooleanBlock(
        required=False,
        default=False,
        verbose_name=_('Show content walls on children pages?'),
        help_text=_(
            'If this is checked, the content walls will be displayed on all children pages of this page.')  # noqa
    )

    class Meta:
        icon = 'fa-stop'
        label = _('Content Wall')
        template = 'coderedcms/blocks/content_wall_block.html'


class ReusableContentBlock(BaseBlock):
    """
    Enables choosing a ResusableContent snippet.
    """
    content = SnippetChooserBlock('coderedcms.ReusableContent')

    class Meta:
        icon = 'fa-recycle'
        label = _('Reusable Content')
        template = 'coderedcms/blocks/reusable_content_block.html'



"""
Custom wagtail settings used by CodeRed CMS.
Settings are user-configurable on a per-site basis (multisite).
Global project or developer settings should be defined in coderedcms.settings.py .
"""

import json
from django.db import models
from django.utils.translation import gettext_lazy as _
from wagtail.admin.edit_handlers import FieldPanel, MultiFieldPanel
from wagtail.images.edit_handlers import ImageChooserPanel
from wagtail.contrib.settings.models import BaseSetting, register_setting
from wagtail.images import get_image_model_string

from coderedcms.fields import MonospaceField
from coderedcms.settings import cr_settings


@register_setting(icon='fa-facebook-official')
class SocialMediaSettings(BaseSetting):
    """
    Social media accounts.
    """
    class Meta:
        verbose_name = _('Social Media')

    facebook = models.URLField(
        blank=True,
        verbose_name=_('Facebook'),
        help_text=_('Your Facebook page URL'),
    )
    twitter = models.URLField(
        blank=True,
        verbose_name=_('Twitter'),
        help_text=_('Your Twitter page URL'),
    )
    instagram = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_('Instagram'),
        help_text=_('Your Instagram username, without the @'),
    )
    youtube = models.URLField(
        blank=True,
        verbose_name=_('YouTube'),
        help_text=_('Your YouTube channel or user account URL'),
    )
    linkedin = models.URLField(
        blank=True,
        verbose_name=_('LinkedIn'),
        help_text=_('Your LinkedIn page URL'),
    )
    googleplus = models.URLField(
        blank=True,
        verbose_name=_('Google'),
        help_text=_('Your Google+ page or Google business listing URL'),
    )

    @property
    def twitter_handle(self):
        """
        Gets the handle of the twitter account from a URL.
        """
        return self.twitter.strip().strip('/').split('/')[-1]

    @property
    def social_json(self):
        """
        Returns non-blank social accounts as a JSON list.
        """
        socialist = [
            self.facebook,
            self.twitter,
            self.instagram,
            self.youtube,
            self.linkedin,
            self.googleplus,
        ]
        socialist = list(filter(None, socialist))
        return json.dumps(socialist)

    panels = [
        MultiFieldPanel(
            [
                FieldPanel('facebook'),
                FieldPanel('twitter'),
                FieldPanel('instagram'),
                FieldPanel('youtube'),
                FieldPanel('linkedin'),
                FieldPanel('googleplus'),
            ],
            _('Social Media Accounts'),
        )
    ]


@register_setting(icon='fa-desktop')
class LayoutSettings(BaseSetting):
    """
    Branding, navbar, and theme settings.
    """
    class Meta:
        verbose_name = _('Layout')

    logo = models.ForeignKey(
        get_image_model_string(),
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name=_('Logo'),
        help_text=_('Brand logo used in the navbar and throughout the site')
    )
    favicon = models.ForeignKey(
        get_image_model_string(),
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='favicon',
        verbose_name=_('Favicon'),
    )
    navbar_color_scheme = models.CharField(
        blank=True,
        max_length=50,
        choices=cr_settings['FRONTEND_NAVBAR_COLOR_SCHEME_CHOICES'],
        default=cr_settings['FRONTEND_NAVBAR_COLOR_SCHEME_DEFAULT'],
        verbose_name=_('Navbar color scheme'),
        help_text=_('Optimizes text and other navbar elements for use with light or dark backgrounds.'),  # noqa
    )
    navbar_class = models.CharField(
        blank=True,
        max_length=255,
        default=cr_settings['FRONTEND_NAVBAR_CLASS_DEFAULT'],
        verbose_name=_('Navbar CSS class'),
        help_text=_('Custom classes applied to navbar e.g. "bg-light", "bg-dark", "bg-primary".'),
    )
    navbar_fixed = models.BooleanField(
        default=False,
        verbose_name=_('Fixed navbar'),
        help_text=_('Fixed navbar will remain at the top of the page when scrolling.'),
    )
    navbar_wrapper_fluid = models.BooleanField(
        default=True,
        verbose_name=_('Full width navbar'),
        help_text=_('The navbar will fill edge to edge.'),
    )
    navbar_content_fluid = models.BooleanField(
        default=False,
        verbose_name=_('Full width navbar contents'),
        help_text=_('Content within the navbar will fill edge to edge.'),
    )
    navbar_collapse_mode = models.CharField(
        blank=True,
        max_length=50,
        choices=cr_settings['FRONTEND_NAVBAR_COLLAPSE_MODE_CHOICES'],
        default=cr_settings['FRONTEND_NAVBAR_COLLAPSE_MODE_DEFAULT'],
        verbose_name=_('Collapse navbar menu'),
        help_text=_('Control on what screen sizes to show and collapse the navbar menu links.'),
    )
    navbar_format = models.CharField(
        blank=True,
        max_length=50,
        choices=cr_settings['FRONTEND_NAVBAR_FORMAT_CHOICES'],
        default=cr_settings['FRONTEND_NAVBAR_FORMAT_DEFAULT'],
        verbose_name=_('Navbar format'),
    )
    navbar_search = models.BooleanField(
        default=True,
        verbose_name=_('Search box'),
        help_text=_('Show search box in navbar')
    )
    frontend_theme = models.CharField(
        blank=True,
        max_length=50,
        choices=cr_settings['FRONTEND_THEME_CHOICES'],
        default=cr_settings['FRONTEND_THEME_DEFAULT'],
        verbose_name=_('Theme variant'),
        help_text=cr_settings['FRONTEND_THEME_HELP'],
    )

    panels = [
        MultiFieldPanel(
            [
                ImageChooserPanel('logo'),
                ImageChooserPanel('favicon'),
            ],
            heading=_('Branding')
        ),
        MultiFieldPanel(
            [
                FieldPanel('navbar_color_scheme'),
                FieldPanel('navbar_class'),
                FieldPanel('navbar_fixed'),
                FieldPanel('navbar_wrapper_fluid'),
                FieldPanel('navbar_content_fluid'),
                FieldPanel('navbar_collapse_mode'),
                FieldPanel('navbar_format'),
                FieldPanel('navbar_search'),
            ],
            heading=_('Site Navbar Layout')
        ),
        MultiFieldPanel(
            [
                FieldPanel('frontend_theme'),
            ],
            heading=_('Theming')
        ),
    ]


@register_setting(icon='fa-google')
class AnalyticsSettings(BaseSetting):
    """
    Tracking and Google Analytics.
    """
    class Meta:
        verbose_name = _('Tracking')

    ga_tracking_id = models.CharField(
        blank=True,
        max_length=255,
        verbose_name=_('GA Tracking ID'),
        help_text=_('Your Google Analytics tracking ID (begins with "UA-")'),
    )
    ga_track_button_clicks = models.BooleanField(
        default=False,
        verbose_name=_('Track button clicks'),
        help_text=_('Track all button clicks using Google Analytics event tracking. Event tracking details can be specified in each button’s advanced settings options.'),  # noqa
    )
    head_scripts = MonospaceField(
        blank=True,
        null=True,
        verbose_name=_('<head> tracking scripts'),
        help_text=_('Add tracking scripts between the <head> tags.'),
    )
    body_scripts = MonospaceField(
        blank=True,
        null=True,
        verbose_name=_('<body> tracking scripts'),
        help_text=_('Add tracking scripts toward closing <body> tag.'),
    )

    panels = [
        MultiFieldPanel(
            [
                FieldPanel('ga_tracking_id'),
                FieldPanel('ga_track_button_clicks'),
            ],
            heading=_('Google Analytics')
        ),
        MultiFieldPanel(
            [
                FieldPanel('head_scripts'),
                FieldPanel('body_scripts'),
            ],
            heading=_('Other Tracking Scripts')
        )
    ]


@register_setting(icon='fa-universal-access')
class ADASettings(BaseSetting):
    """
    Accessibility related options.
    """
    class Meta:
        verbose_name = 'Accessibility'

    skip_navigation = models.BooleanField(
        default=False,
        verbose_name=_('Show skip navigation link'),
        help_text=_('Shows a "Skip Navigation" link above the navbar that takes you directly to the main content.'),  # noqa
    )

    panels = [
        MultiFieldPanel(
            [
                FieldPanel('skip_navigation'),
            ],
            heading=_('Accessibility')
        )
    ]


@register_setting(icon='cog')
class GeneralSettings(BaseSetting):
    """
    Various site-wide settings. A good place to put
    one-off settings that don't belong anywhere else.
    """

    from_email_address = models.CharField(

        blank=True,
        max_length=255,
        verbose_name=_('From email address'),
        help_text=_('The default email address this site appears to send from. For example: "sender@example.com" or "Sender Name <sender@example.com>" (without quotes)'),  # noqa
    )
    search_num_results = models.PositiveIntegerField(
        default=10,
        verbose_name=_('Number of results per page'),
    )
    external_new_tab = models.BooleanField(
        default=False,
        verbose_name=_('Open all external links in new tab')
    )

    panels = [
        MultiFieldPanel(
            [
                FieldPanel('from_email_address'),
            ],
            _('Email')
        ),
        MultiFieldPanel(
            [
                FieldPanel('search_num_results'),
            ],
            _('Search Settings')
        ),
        MultiFieldPanel(
            [
                FieldPanel('external_new_tab'),
            ],
            _('Links')
        ),
    ]

    class Meta:
        verbose_name = _('General')


@register_setting(icon='fa-puzzle-piece')
class GoogleApiSettings(BaseSetting):
    """
    Settings for Google API services.
    """
    class Meta:
        verbose_name = _('Google API')

    google_maps_api_key = models.CharField(
        blank=True,
        max_length=255,
        verbose_name=_('Google Maps API Key'),
        help_text=_('The API Key used for Google Maps.')
    )


@register_setting(icon='fa-puzzle-piece')
class MailchimpApiSettings(BaseSetting):
    """
    Settings for Mailchimp API services.
    """
    class Meta:
        verbose_name = _('Mailchimp API')

    mailchimp_api_key = models.CharField(
        blank=True,
        max_length=255,
        verbose_name=_('Mailchimp API Key'),
        help_text=_('The API Key used for Mailchimp.')
    )



# Generated by Django 2.1.5 on 2019-02-01 20:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coderedcms', '0008_auto_20190122_1242'),
    ]

    operations = [
        migrations.AlterField(
            model_name='coderedpage',
            name='index_order_by',
            field=models.CharField(blank=True, choices=[('', 'Default Ordering'), ('-first_published_at', 'Date first published, newest to oldest'), ('first_published_at', 'Date first published, oldest to newest'), ('-last_published_at', 'Date updated, newest to oldest'), ('last_published_at', 'Date updated, oldest to newest'), ('title', 'Title, alphabetical'), ('-title', 'Title, reverse alphabetical')], default='', max_length=255, verbose_name='Order child pages by'),
        ),
    ]



# Generated by Django 2.2.9 on 2019-12-27 20:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coderedcms', '0016_auto_20191025_1620'),
    ]

    operations = [
        migrations.AddField(
            model_name='generalsettings',
            name='external_new_tab',
            field=models.BooleanField(default=False, verbose_name='Open all external links in new tab'),
        ),
    ]



from django.urls import include, path, re_path
from wagtail.contrib.sitemaps.views import sitemap
from wagtail.core import urls as wagtailcore_urls
from coderedcms.settings import cr_settings
from coderedcms.views import (
    event_generate_ical_for_calendar,
    event_generate_recurring_ical_for_event,
    event_generate_single_ical_for_event,
    event_get_calendar_events,
    favicon,
    robots,
    serve_protected_file
)


urlpatterns = [
    # CodeRed custom URLs
    re_path(r'^favicon\.ico$', favicon, name='codered_favicon'),
    re_path(r'^robots\.txt$', robots, name='codered_robots'),
    re_path(r'^sitemap\.xml$', sitemap, name='codered_sitemap'),
    re_path(r'^{0}(?P<path>.*)$'.format(
        cr_settings['PROTECTED_MEDIA_URL'].lstrip('/')),
        serve_protected_file,
        name="serve_protected_file"
    ),

    # Event/Calendar URLs
    path('ical/generate/single/', event_generate_single_ical_for_event,
         name='event_generate_single_ical'),
    path('ical/generate/recurring/', event_generate_recurring_ical_for_event,
         name='event_generate_recurring_ical'),
    path('ical/generate/calendar/', event_generate_ical_for_calendar,
         name='event_generate_ical_for_calendar'),
    path('ajax/calendar/events/', event_get_calendar_events, name='event_get_calendar_events'),

    # Wagtail
    re_path(r'', include(wagtailcore_urls)),
]



import pytest
import unittest

from ast import literal_eval

from django.urls import reverse
from django.test import Client
from django.test.utils import override_settings

from wagtail.core.models import Site, Page
from wagtail.images.tests.utils import Image, get_test_image_file

from coderedcms.models import LayoutSettings
from coderedcms.tests.testapp.models import EventPage, EventIndexPage, EventOccurrence


@pytest.mark.django_db
class TestSiteURLs(unittest.TestCase):
    def setUp(self):
        self.client = Client()

    @override_settings(DEBUG=False)
    def test_404(self):
        response = self.client.get("/testing/404/page/", follow=True)

        self.assertEqual(response.status_code, 404)

    def test_sitemap(self):
        response = self.client.get("/sitemap.xml")

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['content-type'], 'application/xml')

    def test_robots(self):
        response = self.client.get("/robots.txt")

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['content-type'], 'text/plain')

    def test_search(self):
        response = self.client.get(reverse(
            'codered_search'),
            {'s': 'Test Search Query'},
            follow=True
        )

        self.assertEqual(response.status_code, 200)
        self.assertNotEqual(response.context['results'], None)

        response = self.client.get(reverse(
            'codered_search'),
            {
                's': 'keyword',
                't': 't',
            },
            follow=False
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['results'], None)


@pytest.mark.django_db
class TestEventURLs(unittest.TestCase):
    def setUp(self):
        self.client = Client()
        self.root_page = Page.get_root_nodes()[0]

    def test_generate_single_event(self):
        event_page = EventPage(
            path='/single-event/',
            depth=1,
            title='Single Event',
            slug='single-event'
        )
        self.root_page.add_child(instance=event_page)
        occurrence = EventOccurrence(
            event=event_page,
            start='2019-01-01T10:00:00+0000',
            end='2019-01-01T11:00:00+0000'
        )
        occurrence.save()

        response = self.client.post(
            "/ical/generate/single/",
            {
                'event_pk': event_page.pk,
                'datetime_start': EventOccurrence.objects.get(
                    event=event_page
                ).start.strftime("%Y-%m-%d %H:%M:%S%z").replace(' ', 'T'),
                'datetime_end': EventOccurrence.objects.get(
                    event=event_page
                ).end.strftime("%Y-%m-%d %H:%M:%S%z").replace(' ', 'T'),
            },
            follow=True
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Filename'], "{0}.ics".format(event_page.slug))
        self.assertEqual(
            response['Content-Disposition'],
            'attachment; filename={0}.ics'.format(event_page.slug)
        )
        self.assertEqual(response['content-type'], 'text/calendar')

        # Get datetimes from response and compare them to datetimes on page
        # startswith() is used because older versions of Python
        # use different datetime formatting, specifically for timezones
        split_content = str(response._container[0]).split('VALUE=DATE-TIME:')
        start = split_content[1].split('\\')[0]
        end = split_content[2].split('\\')[0]
        self.assertTrue(
            start.startswith(
                EventOccurrence.objects.get(event=event_page).start.strftime("%Y%m%dT%H%M%S")
            )
        )
        self.assertTrue(
            end.startswith(
                EventOccurrence.objects.get(event=event_page).end.strftime("%Y%m%dT%H%M%S")
            )
        )

    def test_generate_recurring_event(self):
        event_page = EventPage(
            path='/recurring-event/',
            depth=1,
            title='Recurring Event',
            slug='recurring-event'
        )
        self.root_page.add_child(instance=event_page)
        occurrence = EventOccurrence(
            event=event_page,
            start='2019-01-01T10:00:00+0000',
            end='2019-01-01T11:00:00+0000'
        )
        occurrence.save()

        response = self.client.post(
            "/ical/generate/recurring/",
            {'event_pk': event_page.pk},
            follow=True
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Filename'], "{0}.ics".format(event_page.slug))
        self.assertEqual(
            response['Content-Disposition'],
            'attachment; filename={0}.ics'.format(event_page.slug)
        )
        self.assertEqual(response['content-type'], 'text/calendar')

        # Get datetimes from response and compare them to datetimes on page
        # startswith() is used because older versions of Python
        # use different datetime formatting, specifically for timezones
        split_content = str(response._container[0]).split('VALUE=DATE-TIME:')
        start = split_content[1].split('\\')[0]
        end = split_content[2].split('\\')[0]
        self.assertTrue(
            start.startswith(
                EventOccurrence.objects.get(event=event_page).start.strftime("%Y%m%dT%H%M%S")
            )
        )
        self.assertTrue(
            end.startswith(
                EventOccurrence.objects.get(event=event_page).end.strftime("%Y%m%dT%H%M%S")
            )
        )

    def test_generate_calendar(self):
        calendar_page = EventIndexPage(
            path='/event-index-page/',
            depth=1,
            title='Event Index Page',
            slug='event-index-page'
        )
        self.root_page.add_child(instance=calendar_page)

        event_page = EventPage(
            path='/eventpage/1/',
            depth=2,
            title='Event Page 1',
            slug='eventpage1'
        )
        calendar_page.add_child(instance=event_page)
        occurrence = EventOccurrence(
            event=event_page,
            start='2019-01-01T10:00:00+0000',
            end='2019-01-01T11:00:00+0000'
        )
        occurrence.save()

        response = self.client.post(
            "/ical/generate/calendar/",
            {'page_id': calendar_page.pk},
            follow=True
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Filename'], 'calendar.ics')
        self.assertEqual(response['Content-Disposition'], 'attachment; filename=calendar.ics')
        self.assertEqual(response['content-type'], 'text/calendar')

        # Get datetimes from response and compare them to datetimes on page
        # startswith() is used because older versions of Python
        # use different datetime formatting, specifically for timezones
        split_content = str(response._container[0]).split('VALUE=DATE-TIME:')
        start = split_content[1].split('\\')[0]
        end = split_content[2].split('\\')[0]
        self.assertTrue(
            start.startswith(
                EventOccurrence.objects.get(event=event_page).start.strftime("%Y%m%dT%H%M%S")
            )
        )
        self.assertTrue(
            end.startswith(
                EventOccurrence.objects.get(event=event_page).end.strftime("%Y%m%dT%H%M%S")
            )
        )

    def test_ajax_calendar(self):
        calendar_page = EventIndexPage(
            path='/event-index-page/',
            depth=1,
            title='Event Index Page',
            slug='event-index-page'
        )
        self.root_page.add_child(instance=calendar_page)

        event_page = EventPage(
            path='/eventpage/1/',
            depth=2,
            title='Event Page 1',
            slug='eventpage1'
        )
        calendar_page.add_child(instance=event_page)
        occurrence_one = EventOccurrence(
            event=event_page,
            start='2019-01-01T10:00:00+0000',
            end='2019-01-01T11:00:00+0000'
        )
        occurrence_one.save()

        response = self.client.post(
            "/ajax/calendar/events/?pid=" + str(calendar_page.pk),
            follow=True,
            **{'HTTP_X_REQUESTED_WITH': 'XMLHttpRequest'}
        )
        self.assertEqual(response.status_code, 200)

        # Get datetimes from response and compare them to datetimes on page
        start = literal_eval(response._container[0].decode()[1:-1])['start']
        end = literal_eval(response._container[0].decode()[1:-1])['end']
        self.assertEqual(
            start,
            EventOccurrence.objects.get(event=event_page).start.strftime("%Y-%m-%dT%H:%M:%S")
        )
        self.assertEqual(
            end,
            EventOccurrence.objects.get(event=event_page).end.strftime("%Y-%m-%dT%H:%M:%S")
        )


@pytest.mark.django_db
class TestFavicon(unittest.TestCase):
    def test_404(self):
        client = Client()
        # Get the default site
        site = Site.objects.filter(is_default_site=True)[0]
        # Ensure the favicon is blank
        layout = LayoutSettings.for_site(site)
        layout.favicon = None
        layout.save()
        # Expect a 404
        response = client.get("/favicon.ico")
        self.assertEqual(response.status_code, 404)

    def test_301(self):
        client = Client()
        # Get the default site
        site = Site.objects.filter(is_default_site=True)[0]
        # Set a dummy favicon
        layout = LayoutSettings.for_site(site)
        img = Image.objects.create(
            title="Test image",
            file=get_test_image_file(),
        )
        layout.favicon = img
        layout.save()
        # Expect a 301 redirect
        response = client.get("/favicon.ico")
        self.assertEqual(response.status_code, 301)



# Generated by Django 2.1.7 on 2019-03-22 17:24

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('coderedcms', '0011_merge_20190322_1309'),
        ('coderedcms', '0010_auto_20190320_1231'),
    ]

    operations = [
    ]



# Generated by Django 3.1.13 on 2021-07-31 22:53

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailimages', '0023_add_choose_permissions'),
        ('coderedcms', '0021_auto_20210730_1637'),
    ]

    operations = [
        migrations.AddField(
            model_name='coderedpage',
            name='canonical_url',
            field=models.URLField(blank=True, help_text="Leave blank to use the page's URL.", max_length=255, verbose_name='Canonical URL'),
        ),
        migrations.AlterField(
            model_name='coderedpage',
            name='og_image',
            field=models.ForeignKey(blank=True, help_text='Shown when linking to this page on social media.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.image', verbose_name='Preview image'),
        ),
        migrations.AlterField(
            model_name='coderedpage',
            name='struct_org_geo_lat',
            field=models.DecimalField(blank=True, decimal_places=8, max_digits=11, null=True, verbose_name='Geographic latitude'),
        ),
        migrations.AlterField(
            model_name='coderedpage',
            name='struct_org_geo_lng',
            field=models.DecimalField(blank=True, decimal_places=8, max_digits=11, null=True, verbose_name='Geographic longitude'),
        ),
        migrations.DeleteModel(
            name='SeoSettings',
        ),
    ]



"""
Blocks module entry point. Used to cleanly organize blocks into
individual files based on purpose, but provide them all as a
single `blocks` module.
"""

from django.utils.translation import gettext_lazy as _

from wagtail.core import blocks

from .stream_form_blocks import (
    CoderedStreamFormCharFieldBlock,
    CoderedStreamFormCheckboxesFieldBlock,
    CoderedStreamFormCheckboxFieldBlock,
    CoderedStreamFormDateFieldBlock,
    CoderedStreamFormDateTimeFieldBlock,
    CoderedStreamFormDropdownFieldBlock,
    CoderedStreamFormFileFieldBlock,
    CoderedStreamFormImageFieldBlock,
    CoderedStreamFormNumberFieldBlock,
    CoderedStreamFormRadioButtonsFieldBlock,
    CoderedStreamFormStepBlock,
    CoderedStreamFormTextFieldBlock,
    CoderedStreamFormTimeFieldBlock
)
from .html_blocks import (
    ButtonBlock,
    EmbedGoogleMapBlock,
    ImageBlock,
    ImageLinkBlock,
    DownloadBlock,
    EmbedVideoBlock,
    PageListBlock,
    PagePreviewBlock,
    QuoteBlock,
    RichTextBlock,
    TableBlock
)
from .content_blocks import (  # noqa
    CardBlock,
    CarouselBlock,
    ContentWallBlock,
    ImageGalleryBlock,
    ModalBlock,
    NavDocumentLinkWithSubLinkBlock,
    NavExternalLinkWithSubLinkBlock,
    NavPageLinkWithSubLinkBlock,
    PriceListBlock,
    ReusableContentBlock
)
from .layout_blocks import (
    CardGridBlock,
    GridBlock,
    HeroBlock
)
from .base_blocks import (  # noqa
    BaseBlock,
    BaseLayoutBlock,
    BaseLinkBlock,
    ClassifierTermChooserBlock,
    CoderedAdvColumnSettings,
    CoderedAdvSettings,
    CoderedAdvTrackingSettings,
    CollectionChooserBlock,
)

# Collections of blocks commonly used together.

HTML_STREAMBLOCKS = [
    ('text', RichTextBlock(icon='fa-file-text-o')),
    ('button', ButtonBlock()),
    ('image', ImageBlock()),
    ('image_link', ImageLinkBlock()),
    ('html', blocks.RawHTMLBlock(icon='code', form_classname='monospace', label=_('HTML'), )),
    ('download', DownloadBlock()),
    ('embed_video', EmbedVideoBlock()),
    ('quote', QuoteBlock()),
    ('table', TableBlock()),
    ('google_map', EmbedGoogleMapBlock()),
    ('page_list', PageListBlock()),
    ('page_preview', PagePreviewBlock()),
]

CONTENT_STREAMBLOCKS = HTML_STREAMBLOCKS + [
    ('card', CardBlock()),
    ('carousel', CarouselBlock()),
    ('image_gallery', ImageGalleryBlock()),
    ('modal', ModalBlock(HTML_STREAMBLOCKS)),
    ('pricelist', PriceListBlock()),
    ('reusable_content', ReusableContentBlock()),
]

NAVIGATION_STREAMBLOCKS = [
    ('page_link', NavPageLinkWithSubLinkBlock()),
    ('external_link', NavExternalLinkWithSubLinkBlock()),
    ('document_link', NavDocumentLinkWithSubLinkBlock()),
]

BASIC_LAYOUT_STREAMBLOCKS = [
    ('row', GridBlock(HTML_STREAMBLOCKS)),
    ('html', blocks.RawHTMLBlock(icon='code', form_classname='monospace', label=_('HTML'))),
]

LAYOUT_STREAMBLOCKS = [
    ('hero', HeroBlock([
        ('row', GridBlock(CONTENT_STREAMBLOCKS)),
        ('cardgrid', CardGridBlock([
            ('card', CardBlock()),
        ])),
        ('html', blocks.RawHTMLBlock(icon='code', form_classname='monospace', label=_('HTML'))),
    ])),
    ('row', GridBlock(CONTENT_STREAMBLOCKS)),
    ('cardgrid', CardGridBlock([
        ('card', CardBlock()),
    ])),
    ('html', blocks.RawHTMLBlock(icon='code', form_classname='monospace', label=_('HTML'))),
]

STREAMFORM_FIELDBLOCKS = [
    ('sf_singleline', CoderedStreamFormCharFieldBlock(group=_('Fields'))),
    ('sf_multiline', CoderedStreamFormTextFieldBlock(group=_('Fields'))),
    ('sf_number', CoderedStreamFormNumberFieldBlock(group=_('Fields'))),
    ('sf_checkboxes', CoderedStreamFormCheckboxesFieldBlock(group=_('Fields'))),
    ('sf_radios', CoderedStreamFormRadioButtonsFieldBlock(group=_('Fields'))),
    ('sf_dropdown', CoderedStreamFormDropdownFieldBlock(group=_('Fields'))),
    ('sf_checkbox', CoderedStreamFormCheckboxFieldBlock(group=_('Fields'))),
    ('sf_date', CoderedStreamFormDateFieldBlock(group=_('Fields'))),
    ('sf_time', CoderedStreamFormTimeFieldBlock(group=_('Fields'))),
    ('sf_datetime', CoderedStreamFormDateTimeFieldBlock(group=_('Fields'))),
    ('sf_image', CoderedStreamFormImageFieldBlock(group=_('Fields'))),
    ('sf_file', CoderedStreamFormFileFieldBlock(group=_('Fields'))),
]

STREAMFORM_BLOCKS = [
    ('step', CoderedStreamFormStepBlock(STREAMFORM_FIELDBLOCKS + HTML_STREAMBLOCKS)),
]
